'use strict';

/*
 * ACHTUNG: there is a copied & modified version of this file in
 * <basedir>/src/tests/container/specs/api/pad.js
 *
 * TODO: unify those two files, and merge in a single one.
 */

const assert = require('assert').strict;
const common = require('../../common');

let agent;
const apiKey = common.apiKey;
let apiVersion = 1;
const testPadId = makeid();
let lastEdited = '';
const text = generateLongText();

const endPoint = (point, version) => `/api/${version || apiVersion}/${point}?apikey=${apiKey}`;

/*
 * Html document with nested lists of different types, to test its import and
 * verify it is exported back correctly
 */
const ulHtml = '<!doctype html><html><body><ul class="bullet"><li>one</li><li>two</li><li>0</li><li>1</li><li>2<ul class="bullet"><li>3</li><li>4</li></ul></li></ul><ol class="number"><li>item<ol class="number"><li>item1</li><li>item2</li></ol></li></ol></body></html>';

/*
 * When exported back, Etherpad produces an html which is not exactly the same
 * textually, but at least it remains standard compliant and has an equal DOM
 * structure.
 */
const expectedHtml = '<!doctype html><html><body><ul class="bullet"><li>one</li><li>two</li><li>0</li><li>1</li><li>2<ul class="bullet"><li>3</li><li>4</ul></li></ul><ol start="1" class="number"><li>item<ol start="2" class="number"><li>item1</li><li>item2</ol></li></ol></body></html>';

/*
 * Html document with space between list items, to test its import and
 * verify it is exported back correctly
 */
const ulSpaceHtml = '<!doctype html><html><body><ul class="bullet"> <li>one</li></ul></body></html>';

/*
 * When exported back, Etherpad produces an html which is not exactly the same
 * textually, but at least it remains standard compliant and has an equal DOM
 * structure.
 */
const expectedSpaceHtml = '<!doctype html><html><body><ul class="bullet"><li>one</ul></body></html>';

describe(__filename, function () {
  before(async function () { agent = await common.init(); });

  describe('Connectivity', function () {
    it('can connect', async function () {
      await agent.get('/api/')
          .expect(200)
          .expect('Content-Type', /json/);
    });
  });

  describe('API Versioning', function () {
    it('finds the version tag', async function () {
      const res = await agent.get('/api/')
          .expect(200);
      apiVersion = res.body.currentVersion;
      if (!apiVersion) throw new Error('No version set in API');
    });
  });

  describe('Permission', function () {
    it('errors with invalid APIKey', async function () {
      // This is broken because Etherpad doesn't handle HTTP codes properly see #2343
      // If your APIKey is password you deserve to fail all tests anyway
      await agent.get(`/api/${apiVersion}/createPad?apikey=password&padID=test`)
          .expect(401);
    });
  });

  /* Pad Tests Order of execution
  -> deletePad -- This gives us a guaranteed clear environment
   -> createPad
    -> getRevisions -- Should be 0
     -> getSavedRevisionsCount(padID) -- Should be 0
      -> listSavedRevisions(padID) -- Should be an empty array
       -> getHTML -- Should be the default pad text in HTML format
        -> deletePad -- Should just delete a pad
         -> getHTML -- Should return an error
          -> createPad(withText)
           -> getText -- Should have the text specified above as the pad text
            -> setText
             -> getText -- Should be the text set before
              -> getRevisions -- Should be 0 still?
               -> saveRevision
                -> getSavedRevisionsCount(padID) -- Should be 0 still?
                 -> listSavedRevisions(padID) -- Should be an empty array still ?
                  -> padUsersCount -- Should be 0
                   -> getReadOnlyId -- Should be a value
                    -> listAuthorsOfPad(padID) -- should be empty array?
                     -> getLastEdited(padID) -- Should be when pad was made
                      -> setText(padId)
                       -> getLastEdited(padID) -- Should be when setText was performed
                        -> padUsers(padID) -- Should be when setText was performed

                         -> setText(padId, "hello world")
                          -> getLastEdited(padID) -- Should be when pad was made
                           -> getText(padId) -- Should be "hello world"
                            -> movePad(padID, newPadId) -- Should provide consistent pad data
                             -> getText(newPadId) -- Should be "hello world"
                              -> movePad(newPadID, originalPadId) -- Should provide consistent pad data
                               -> getText(originalPadId) -- Should be "hello world"
                                -> getLastEdited(padID) -- Should not be 0
                                -> appendText(padID, "hello")
                                -> getText(padID) -- Should be "hello worldhello"
                                 -> setHTML(padID) -- Should fail on invalid HTML
                                  -> setHTML(padID) *3 -- Should fail on invalid HTML
                                   -> getHTML(padID) -- Should return HTML close to posted HTML
                                    -> createPad -- Tries to create pads with bad url characters

  */

  describe('deletePad', function () {
    it('deletes a Pad', async function () {
      await agent.get(`${endPoint('deletePad')}&padID=${testPadId}`)
          .expect(200) // @TODO: we shouldn't expect 200 here since the pad may not exist
          .expect('Content-Type', /json/);
    });
  });

  describe('createPad', function () {
    it('creates a new Pad', async function () {
      const res = await agent.get(`${endPoint('createPad')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to create new Pad');
    });
  });

  describe('getRevisionsCount', function () {
    it('gets revision count of Pad', async function () {
      const res = await agent.get(`${endPoint('getRevisionsCount')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to get Revision Count');
      if (res.body.data.revisions !== 0) throw new Error('Incorrect Revision Count');
    });
  });

  describe('getSavedRevisionsCount', function () {
    it('gets saved revisions count of Pad', async function () {
      const res = await agent.get(`${endPoint('getSavedRevisionsCount')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to get Saved Revisions Count');
      if (res.body.data.savedRevisions !== 0) throw new Error('Incorrect Saved Revisions Count');
    });
  });

  describe('listSavedRevisions', function () {
    it('gets saved revision list of Pad', async function () {
      const res = await agent.get(`${endPoint('listSavedRevisions')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to get Saved Revisions List');
      assert.deepEqual(res.body.data.savedRevisions, []);
    });
  });

  describe('getHTML', function () {
    it('get the HTML of Pad', async function () {
      const res = await agent.get(`${endPoint('getHTML')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.html.length <= 1) throw new Error('Unable to get the HTML');
    });
  });

  describe('listAllPads', function () {
    it('list all pads', async function () {
      const res = await agent.get(endPoint('listAllPads'))
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.padIDs.includes(testPadId) !== true) {
        throw new Error('Unable to find pad in pad list');
      }
    });
  });

  describe('deletePad', function () {
    it('deletes a Pad', async function () {
      const res = await agent.get(`${endPoint('deletePad')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Deletion failed');
    });
  });

  describe('listAllPads', function () {
    it('list all pads', async function () {
      const res = await agent.get(endPoint('listAllPads'))
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.padIDs.includes(testPadId) !== false) {
        throw new Error('Test pad should not be in pads list');
      }
    });
  });

  describe('getHTML', function () {
    it('get the HTML of a Pad -- Should return a failure', async function () {
      const res = await agent.get(`${endPoint('getHTML')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 1) throw new Error('Pad deletion failed');
    });
  });

  describe('createPad', function () {
    it('creates a new Pad with text', async function () {
      const res = await agent.get(`${endPoint('createPad')}&padID=${testPadId}&text=testText`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Creation failed');
    });
  });

  describe('getText', function () {
    it('gets the Pad text and expect it to be testText with \n which is a line break', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.text !== 'testText\n') throw new Error('Pad Creation with text');
    });
  });

  describe('setText', function () {
    it('creates a new Pad with text', async function () {
      const res = await agent.post(endPoint('setText'))
          .send({
            padID: testPadId,
            text: 'testTextTwo',
          })
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad setting text failed');
    });
  });

  describe('getText', function () {
    it('gets the Pad text', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.text !== 'testTextTwo\n') throw new Error('Setting Text');
    });
  });

  describe('getRevisionsCount', function () {
    it('gets Revision Count of a Pad', async function () {
      const res = await agent.get(`${endPoint('getRevisionsCount')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.revisions !== 1) throw new Error('Unable to get text revision count');
    });
  });

  describe('saveRevision', function () {
    it('saves Revision', async function () {
      const res = await agent.get(`${endPoint('saveRevision')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to save Revision');
    });
  });

  describe('getSavedRevisionsCount', function () {
    it('gets saved revisions count of Pad', async function () {
      const res = await agent.get(`${endPoint('getSavedRevisionsCount')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to get Saved Revisions Count');
      if (res.body.data.savedRevisions !== 1) {
        throw new Error('Incorrect Saved Revisions Count');
      }
    });
  });

  describe('listSavedRevisions', function () {
    it('gets saved revision list of Pad', async function () {
      const res = await agent.get(`${endPoint('listSavedRevisions')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Unable to get Saved Revisions List');
      assert.deepEqual(res.body.data.savedRevisions, [1]);
    });
  });

  describe('padUsersCount', function () {
    it('gets User Count of a Pad', async function () {
      const res = await agent.get(`${endPoint('padUsersCount')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.padUsersCount !== 0) throw new Error('Incorrect Pad User count');
    });
  });

  describe('getReadOnlyID', function () {
    it('Gets the Read Only ID of a Pad', async function () {
      const res = await agent.get(`${endPoint('getReadOnlyID')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (!res.body.data.readOnlyID) throw new Error('No Read Only ID for Pad');
    });
  });

  describe('listAuthorsOfPad', function () {
    it('Get Authors of the Pad', async function () {
      const res = await agent.get(`${endPoint('listAuthorsOfPad')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.authorIDs.length !== 0) {
        throw new Error('# of Authors of pad is not 0');
      }
    });
  });

  describe('getLastEdited', function () {
    it('Get When Pad was left Edited', async function () {
      const res = await agent.get(`${endPoint('getLastEdited')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (!res.body.data.lastEdited) {
        throw new Error('# of Authors of pad is not 0');
      } else {
        lastEdited = res.body.data.lastEdited;
      }
    });
  });

  describe('setText', function () {
    it('creates a new Pad with text', async function () {
      const res = await agent.post(endPoint('setText'))
          .send({
            padID: testPadId,
            text: 'testTextTwo',
          })
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad setting text failed');
    });
  });

  describe('getLastEdited', function () {
    it('Get When Pad was left Edited', async function () {
      const res = await agent.get(`${endPoint('getLastEdited')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.lastEdited <= lastEdited) {
        throw new Error('Editing A Pad is not updating when it was last edited');
      }
    });
  });

  describe('padUsers', function () {
    it('gets User Count of a Pad', async function () {
      const res = await agent.get(`${endPoint('padUsers')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.padUsers.length !== 0) throw new Error('Incorrect Pad Users');
    });
  });

  describe('deletePad', function () {
    it('deletes a Pad', async function () {
      const res = await agent.get(`${endPoint('deletePad')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Deletion failed');
    });
  });

  const newPadId = makeid();
  const copiedPadId = makeid();

  describe('createPad', function () {
    it('creates a new Pad with text', async function () {
      const res = await agent.get(`${endPoint('createPad')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Creation failed');
    });
  });

  describe('setText', function () {
    it('Sets text on a pad Id', async function () {
      const res = await agent.post(`${endPoint('setText')}&padID=${testPadId}`)
          .field({text})
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Set Text failed');
    });
  });

  describe('getText', function () {
    it('Gets text on a pad Id', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Get Text failed');
      if (res.body.data.text !== `${text}\n`) throw new Error('Pad Text not set properly');
    });
  });

  describe('setText', function () {
    it('Sets text on a pad Id including an explicit newline', async function () {
      const res = await agent.post(`${endPoint('setText')}&padID=${testPadId}`)
          .field({text: `${text}\n`})
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Set Text failed');
    });
  });

  describe('getText', function () {
    it("Gets text on a pad Id and doesn't have an excess newline", async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Get Text failed');
      if (res.body.data.text !== `${text}\n`) throw new Error('Pad Text not set properly');
    });
  });

  describe('getLastEdited', function () {
    it('Gets when pad was last edited', async function () {
      const res = await agent.get(`${endPoint('getLastEdited')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.lastEdited === 0) throw new Error('Get Last Edited Failed');
    });
  });

  describe('movePad', function () {
    it('Move a Pad to a different Pad ID', async function () {
      const res = await agent.get(
          `${endPoint('movePad')}&sourceID=${testPadId}&destinationID=${newPadId}&force=true`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Moving Pad Failed');
    });
  });

  describe('getText', function () {
    it('Gets text on a pad Id', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${newPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.text !== `${text}\n`) throw new Error('Pad Get Text failed');
    });
  });

  describe('movePad', function () {
    it('Move a Pad to a different Pad ID', async function () {
      const res = await agent.get(
          `${endPoint('movePad')}&sourceID=${newPadId}&destinationID=${testPadId}&force=false`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Moving Pad Failed');
    });
  });

  describe('getText', function () {
    it('Gets text on a pad Id', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.data.text !== `${text}\n`) throw new Error('Pad Get Text failed');
    });
  });

  describe('getLastEdited', function () {
    it('Gets when pad was last edited', async function () {
      const res = await agent.get(`${endPoint('getLastEdited')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.lastEdited === 0) throw new Error('Get Last Edited Failed');
    });
  });

  describe('appendText', function () {
    it('Append text to a pad Id', async function () {
      const res = await agent.get(
          `${endPoint('appendText', '1.2.13')}&padID=${testPadId}&text=hello`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Append Text failed');
    });
  });

  describe('getText', function () {
    it('Gets text on a pad Id', async function () {
      const res = await agent.get(`${endPoint('getText')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Pad Get Text failed');
      if (res.body.data.text !== `${text}hello\n`) {
        throw new Error('Pad Text not set properly');
      }
    });
  });

  describe('setHTML', function () {
    it('Sets the HTML of a Pad attempting to pass ugly HTML', async function () {
      const html = '<div><b>Hello HTML</title></head></div>';
      const res = await agent.post(endPoint('setHTML'))
          .send({
            padID: testPadId,
            html,
          })
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) {
        throw new Error("Crappy HTML Can't be Imported[we weren't able to sanitize it']");
      }
    });
  });

  describe('setHTML', function () {
    it('Sets the HTML of a Pad with complex nested lists of different types', async function () {
      const res = await agent.post(endPoint('setHTML'))
          .send({
            padID: testPadId,
            html: ulHtml,
          })
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('List HTML cant be imported');
    });
  });

  describe('getHTML', function () {
    it('Gets back the HTML of a Pad with complex nested lists of different types', async function () {
      const res = await agent.get(`${endPoint('getHTML')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      const receivedHtml = res.body.data.html.replace('<br></body>', '</body>').toLowerCase();
      if (receivedHtml !== expectedHtml) {
        throw new Error(`HTML received from export is not the one we were expecting.
           Received:
           ${receivedHtml}

           Expected:
           ${expectedHtml}

           Which is a slightly modified version of the originally imported one:
           ${ulHtml}`);
      }
    });
  });

  describe('setHTML', function () {
    it('Sets the HTML of a Pad with white space between list items', async function () {
      const res = await agent.get(`${endPoint('setHTML')}&padID=${testPadId}&html=${ulSpaceHtml}`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('List HTML cant be imported');
    });
  });

  describe('getHTML', function () {
    it('Gets back the HTML of a Pad with complex nested lists of different types', async function () {
      const res = await agent.get(`${endPoint('getHTML')}&padID=${testPadId}`)
          .expect(200)
          .expect('Content-Type', /json/);
      const receivedHtml = res.body.data.html.replace('<br></body>', '</body>').toLowerCase();
      if (receivedHtml !== expectedSpaceHtml) {
        throw new Error(`HTML received from export is not the one we were expecting.
           Received:
           ${receivedHtml}

           Expected:
           ${expectedSpaceHtml}

           Which is a slightly modified version of the originally imported one:
           ${ulSpaceHtml}`);
      }
    });
  });

  describe('createPad', function () {
    it('errors if pad can be created', async function () {
      await Promise.all(['/', '%23', '%3F', '%26'].map(async (badUrlChar) => {
        const res = await agent.get(`${endPoint('createPad')}&padID=${badUrlChar}`)
            .expect('Content-Type', /json/);
        if (res.body.code !== 1) throw new Error('Pad with bad characters was created');
      }));
    });
  });

  describe('copyPad', function () {
    it('copies the content of a existent pad', async function () {
      const res = await agent.get(
          `${endPoint('copyPad')}&sourceID=${testPadId}&destinationID=${copiedPadId}&force=true`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Copy Pad Failed');
    });
  });

  describe('copyPadWithoutHistory', function () {
    const sourcePadId = makeid();
    let newPad;

    before(async function () {
      await createNewPadWithHtml(sourcePadId, ulHtml);
    });

    beforeEach(async function () {
      newPad = makeid();
    });

    it('returns a successful response', async function () {
      const res = await agent.get(`${endPoint('copyPadWithoutHistory')}&sourceID=${sourcePadId}` +
                                  `&destinationID=${newPad}&force=false`)
          .expect(200)
          .expect('Content-Type', /json/);
      if (res.body.code !== 0) throw new Error('Copy Pad Without History Failed');
    });

    // this test validates if the source pad's text and attributes are kept
    it('creates a new pad with the same content as the source pad', async function () {
      let res = await agent.get(`${endPoint('copyPadWithoutHistory')}&sourceID=${sourcePadId}` +
                                `&destinationID=${newPad}&force=false`);
      if (res.body.code !== 0) throw new Error('Copy Pad Without History Failed');
      res = await agent.get(`${endPoint('getHTML')}&padID=${newPad}`)
          .expect(200);
      const receivedHtml = res.body.data.html.replace('<br><br></body>', '</body>').toLowerCase();
      if (receivedHtml !== expectedHtml) {
        throw new Error(`HTML received from export is not the one we were expecting.
                 Received:
                 ${receivedHtml}

                 Expected:
                 ${expectedHtml}

                 Which is a slightly modified version of the originally imported one:
                 ${ulHtml}`);
      }
    });

    context('when try copy a pad with a group that does not exist', function () {
      const padId = makeid();
      const padWithNonExistentGroup = `notExistentGroup$${padId}`;
      it('throws an error', async function () {
        const res = await agent.get(`${endPoint('copyPadWithoutHistory')}` +
                                    `&sourceID=${sourcePadId}` +
                                    `&destinationID=${padWithNonExistentGroup}&force=true`)
            .expect(200);
        // code 1, it means an error has happened
        if (res.body.code !== 1) throw new Error('It should report an error');
      });
    });

    context('when try copy a pad and destination pad already exist', function () {
      const padIdExistent = makeid();

      before(async function () {
        await createNewPadWithHtml(padIdExistent, ulHtml);
      });

      context('and force is false', function () {
        it('throws an error', async function () {
          const res = await agent.get(`${endPoint('copyPadWithoutHistory')}` +
                                      `&sourceID=${sourcePadId}` +
                                      `&destinationID=${padIdExistent}&force=false`)
              .expect(200);
          // code 1, it means an error has happened
          if (res.body.code !== 1) throw new Error('It should report an error');
        });
      });

      context('and force is true', function () {
        it('returns a successful response', async function () {
          const res = await agent.get(`${endPoint('copyPadWithoutHistory')}` +
                                      `&sourceID=${sourcePadId}` +
                                      `&destinationID=${padIdExistent}&force=true`)
              .expect(200);
          // code 1, it means an error has happened
          if (res.body.code !== 0) {
            throw new Error('Copy pad without history with force true failed');
          }
        });
      });
    });
  });
});

/*
                          -> movePadForce Test

*/

const createNewPadWithHtml = async (padId, html) => {
  await agent.get(`${endPoint('createPad')}&padID=${padId}`);
  await agent.post(endPoint('setHTML'))
      .send({
        padID: padId,
        html,
      });
};

function makeid() {
  let text = '';
  const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (let i = 0; i < 5; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }
  return text;
}

function generateLongText() {
  let text = '';
  const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (let i = 0; i < 80000; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }
  return text;
}
